<?php
/**
 * Robot Ninja API Class
 *
 * @author 	Prospress
 * @since 	1.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class RN_API {

	/**
	 * Init Robot Ninja API Class
	 *
	 * @since 1.0
	 */
	public static function init() {
		// Register our custom routes
		add_action( 'rest_api_init', __CLASS__ . '::register_custom_routes' );

		// Register additional fields to be added to the GET system status endpoint
		add_action( 'rest_api_init', __CLASS__ . '::register_system_status_field', 15 );
	}

	/**
	 * Register Custom Routes
	 *
	 * @since 1.0
	 */
	public static function register_custom_routes() {
		// A simple GET route we can check quickly to determine if the plugin is activated as part of service discovery/onboarding
		register_rest_route( 'rn/helper', '/status', array(
			'methods'  => \WP_REST_Server::READABLE,
			'callback' => __CLASS__ . '::return_plugin_status',
		) );
	}

	/**
	 * Register the additional fields for the WC System Status endpoint.
	 *
	 * @since 1.0
	 */
	public static function register_system_status_field() {
		register_rest_field( 'system_status',
			'robot_ninja_data',
			array(
				'get_callback'    => __CLASS__ . '::add_robot_ninja_data',
				'update_callback' => null,
				'schema'          => null,
			)
		);
	}

	/**
	 * Add additional field to the system status endpoint response for robot ninja
	 *
	 * @since 1.0
	 * @param mixed $response
	 * @param string $field_name
	 * @param WP_Rest_Request
	 * @return Object
	 */
	public static function add_robot_ninja_data( $response, $field_name, $request ) {
		// add some products for robot ninja to test
		$info = new stdClass();
		$info->guest_checkout_enabled = ( 'yes' == get_option( 'woocommerce_enable_guest_checkout' ) ) ? true : false;
		$info->shop_page_display      = ( '' === get_option( 'woocommerce_shop_page_display' ) ) ? 'products' : get_option( 'woocommerce_shop_page_display' );
		$info->products = $info->pages = array();

		$product_visibility_terms = wc_get_product_visibility_term_ids();

		// get two of the most popular products with a price above 0.
		$popular_products = new WP_Query(
			array(
				'post_type'      => 'product',
				'post_status'    => 'publish',
				'meta_key'       => 'total_sales',
				'orderby'        => 'meta_value_num',
				'posts_per_page' => 2,
				'fields'         => 'ids',
				'no_found_rows'  => true,
				'meta_query'     => array(
					'relation'      => 'AND',
					array(
						'key'       => '_regular_price',
						'value'     => 0,
						'compare'   => '>',
					),
					array(
						'key'       => '_stock_status',
						'value'     => 'instock',
						'compare'   => '=',
					),
				),
				'tax_query' => array(
					array( // don't return products set to hidden or excluded from catalog and are out of stock
						'taxonomy' => 'product_visibility',
						'field'    => 'term_taxonomy_id',
						'terms'    => array( $product_visibility_terms['exclude-from-catalog'], $product_visibility_terms['outofstock'] ),
						'operator' => 'NOT IN',
					),
				),
			)
		);

		foreach ( $popular_products->posts as $product_id ) {
			$info->products[ $product_id ] = get_permalink( $product_id );
		}

		$i = 0;
		foreach ( array( 'shop', 'cart', 'checkout', 'my_account' ) as $page_key ) {
			$page_info = $response['pages'][ $i ];

			// only attach the page URLs if the page exists and is visible
			if ( $page_info['page_set'] && $page_info['page_exists'] && $page_info['page_visible'] ) {
				$info->pages[ $page_key ] = $response['environment']->site_url . '/?p=' . $page_info['page_id'];
			}
			$i++;
		}

		return $info;
	}

	/**
	 * Simple callback returning that the plugin is activated
	 *
	 * @since  1.0
	 * @param  WP_Rest_Request $request
	 * @return WP_REST_Response
	 */
	public static function return_plugin_status( $request ) {

		$data = array(
			'status' => 'activated',
			'php_auth_user' => false,
			'php_auth_pw' => false,
			'http_authorization' => false,
			'redirect_http_authorization' => false,
		);

		if ( ! empty( $_SERVER['PHP_AUTH_USER'] ) ) {
			$data['php_auth_user'] = true;
		}

		if ( ! empty( $_SERVER['PHP_AUTH_PW'] ) ) {
			$data['php_auth_pw'] = true;
		}

		if ( ! empty( $_SERVER['HTTP_AUTHORIZATION'] ) ) {
			$data['http_authorization'] = true;
		}

		if ( ! empty( $_SERVER['REDIRECT_HTTP_AUTHORIZATION'] ) ) {
			$data['redirect_http_authorization'] = true;
		}

		return new WP_REST_Response( $data, 200 );
	}
}
RN_API::init();
